#!/bin/bash
### BEGIN INIT INFO
# Provides:          bitmonero-init.sh
# Required-Start:    networking
# Required-Stop:     networking
# Should-Start:      sshd tor
# Should-Stop:       sshd tor
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: monerod init script
# Description:       This file should be used to construct scripts to be
#                    placed in /etc/init.d.
### END INIT INFO

PATH=/sbin:/usr/sbin:/bin:/usr/bin
DESC="Init monerod"
NAME=bitmonero-init.sh
DAEMON=/usr/lib/bitmonero/monerod
MONERO_CLI=/usr/lib/bitmonero/monero-rpc-stop.sh
DAEMON_ARGS=""
SCRIPTNAME=/etc/init.d/$(/usr/bin/basename $0)
DEFAULT_CONF=/etc/default/bitmonero-init

. /lib/init/vars.sh
. /lib/lsb/init-functions

# Exit if the package is not installed
[ -x "$DAEMON" ] || exit 0

if [ -f ${DEFAULT_CONF} ]
then
    . ${DEFAULT_CONF}
fi

set -e

LOG_FILE=`/bin/cat ${SERVER_CONF} | /bin/grep "log-file=" | /usr/bin/mawk -F "=" '{print $2 ;}'`
TESTNET_LOG_FILE=`/bin/cat ${TESTNET_SERVER_CONF} | /bin/grep "log-file=" | /usr/bin/mawk -F "=" '{print $2 ;}'`

if [ -z "${RUN_VERSION}" ]
then
    RUN_VERSION=main
fi

if ! [ "${ENABLE}" = "yes" ]
then
    log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
    exit 0
fi

COMMAND_RUN="${DAEMON} --config-file ${SERVER_CONF} --detach --non-interactive"
COMMAND_RUN_TESTNET="${DAEMON} --config-file ${TESTNET_SERVER_CONF} --detach --non-interactive"

wait_pid_stopped()
{
    PID_WAIT=$1
    IS_RUNNING="1"
    WAIT_COUNTER=0
    WAIT_LIMIT=300

    if [ -n "${PID_WAIT}" ]
    then
	while ! [ "${IS_RUNNING}" = "0" ]
	do
	    IS_RUNNING=`/usr/bin/ps -p ${PID_WAIT} 1>/dev/null 2>/dev/null`
	    IS_RUNNING=$?
	    if [ ${WAIT_COUNTER} -gt ${WAIT_LIMIT} ]
	    then
		IS_RUNNING="0"
	    fi
	    WAIT_COUNTER=$[ ${WAIT_COUNTER} + 1 ]
	    /usr/bin/sleep 1
	done
    fi

    return ${IS_RUNNING}
}

#
# Function that starts the daemon/service
#
do_start()
{
    INIT_TYPE="$1"
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	/usr/bin/sleep ${INIT_DELAY_DAEMON}

	case "${RUN_VERSION}" in
	    main)
		/bin/echo -n "" > ${LOG_FILE}
		/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE}
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${COMMAND_RUN} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
	    ;;
	    testnet)
		/bin/echo -n "" > ${TESTNET_LOG_FILE}
		/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE}
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${COMMAND_RUN_TESTNET} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
	    ;;
	    both)
		/bin/echo -n "" > ${LOG_FILE}
		/bin/echo -n "" > ${TESTNET_LOG_FILE}
		/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE}
		/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE}

		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${COMMAND_RUN} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${COMMAND_RUN_TESTNET} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$[ ${RESULT} + $? ]
	    ;;
	esac
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that stops the daemon/service
#
do_stop()
{
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	case "${RUN_VERSION}" in
	    main)
		PIDO=`/usr/bin/pgrep -f "${COMMAND_RUN}" 2>/dev/null`
		/bin/su ${RUNUSER} -c "${MONERO_CLI} 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		wait_pid_stopped ${PIDO}
	    ;;
	    testnet)
		TESTNET_PIDO=`/usr/bin/pgrep -f "${COMMAND_RUN_TESTNET}" 2>/dev/null`
		/bin/su ${RUNUSER} -c "${MONERO_CLI} testnet 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		wait_pid_stopped ${TESTNET_PIDO}
	    ;;
	    both)
		PIDO=`/usr/bin/pgrep -f "${COMMAND_RUN}" 2>/dev/null`
		TESTNET_PIDO=`/usr/bin/pgrep -f "${COMMAND_RUN_TESTNET}" 2>/dev/null`
		/bin/su ${RUNUSER} -c "${MONERO_CLI} 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		/bin/su ${RUNUSER} -c "${MONERO_CLI} testnet 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$[ ${RESULT} + $? ]
		wait_pid_stopped ${PIDO}
		wait_pid_stopped ${TESTNET_PIDO}
	    ;;
	esac

	COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	COUNTER=0
	while [ ${COUNT_PROC} -gt 0 ]
	do
	    COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	    COUNTER=$[ ${COUNTER} + 1 ]
	    if [ ${COUNTER} -ge 95 ]
	    then
		COUNT_PROC=0
	    fi
	    /bin/sleep 1
	done
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

do_restart()
{
    do_stop
    /bin/sleep 1
    do_start
}

RESULT=0

case "$1" in
  start)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start monerod"
	do_start
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  stop)
	log_daemon_msg "${NAME}"
	log_progress_msg "Stop monerod"
	do_stop
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  restart)
	log_daemon_msg "${NAME}"
	log_progress_msg "Restart monerod"
	do_restart
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  *)
	echo "Usage: $SCRIPTNAME {start|stop|restart}" >&2
	exit 3
	;;
esac

exit ${RESULT}
